import { Transformable } from "../../../common/classes/Transformable";
import { LadderCharacter } from "./LadderCharacter";
import { LadderEntry } from "./LadderEntry";
import { ILadderOptions } from "./models/Options";
/**
 * Each ladder only contains the top 15000 entries.
 * Attempting to fetch entries beyond this will return an empty result.
 */
export declare abstract class Ladder extends Transformable {
    protected ladderOptions: ILadderOptions;
    protected leagueId: string;
    total: number;
    cachedSince: Date;
    entries: LadderEntry[];
    set options(options: ILadderOptions);
    set league(league: string);
    /**
     * @remarks
     * Uses the same [[ILadderOptions]] that have been used getting this ladder.
     * If you want to fetch an entire ladder, it is highly recommended to set the `limit` option to `200`.
     *
     * @param append Set to `true` if the next entries should be appended to this ladders entries
     * @returns The next set of ladder entries, `null` when there are no more entries
     */
    abstract getNextEntries(append: boolean): Promise<LadderEntry[] | null>;
    /**
     * @example
     * Get all characters which have the Assassin ascendancy.
     *
     * ```typescript
     * const assassins = ladder.filterByCharacter("class", "Assassin");
     * console.log(`${assassins.length} of the first 200 characters are Assassins.`);
     * ```
     *
     * @param property Name of a character property
     * @param value Value of the property that should be filtered
     * @returns A new list of characters matching the filter
     */
    filterByCharacter<K extends keyof LadderCharacter>(property: K, value: LadderCharacter[K]): LadderEntry[];
    /**
     * @example
     * Display online characters from the ladder.
     *
     * ```typescript
     * const online = ladder.filterBy("online", true);
     * console.log(`${online.length} of the first 200 characters are currently online.`);
     * ```
     *
     * @param property Name of a ladder entry property
     * @param value Value of the property that should be filtered
     * @returns A new list of characters matching the filter
     */
    filterBy<K extends keyof LadderEntry>(property: K, value: LadderEntry[K]): LadderEntry[];
}
